@@ -31,18 +31,11 @@
 #define PY_SSIZE_T_CLEAN
 #include <Python.h>
 
-#include "config.h"
 #include "private.h"
 
 
 /* class cairo.FontFace --------------------------------------------------- */
 
-/* PycairoFontFace_FromFontFace
- * Create a new PycairoFontFace from a cairo_font_face_t
- * font_face - a cairo_font_face_t to 'wrap' into a Python object.
- *             it is unreferenced if the PycairoFontFace creation fails
- * Return value: New reference or NULL on failure
- */
 PyObject *
 PycairoFontFace_FromFontFace (cairo_font_face_t *font_face) {
   PyTypeObject *type = NULL;
@@ -99,6 +92,27 @@ static PyMethodDef font_face_methods[] = {
 };
 */
 
+
+static PyObject*
+font_face_richcompare (PyObject *self, PyObject *other, int op)
+{
+  if (Py_TYPE(self) == Py_TYPE(other))
+    return Pycairo_richcompare (
+      ((PycairoFontFace *)self)->font_face,
+      ((PycairoFontFace *)other)->font_face,
+      op);
+  else {
+    Py_INCREF(Py_NotImplemented);
+    return Py_NotImplemented;
+  }
+}
+
+static PYCAIRO_Py_hash_t
+font_face_hash (PyObject *self)
+{
+  return PYCAIRO_Py_hash_t_FromVoidPtr (((PycairoFontFace *)self)->font_face);
+}
+
 PyTypeObject PycairoFontFace_Type = {
   PyVarObject_HEAD_INIT(NULL, 0)
   "cairo.FontFace",                   /* tp_name */
@@ -113,7 +127,7 @@ PyTypeObject PycairoFontFace_Type = {
   0,                                  /* tp_as_number */
   0,                                  /* tp_as_sequence */
   0,                                  /* tp_as_mapping */
-  0,                                  /* tp_hash */
+  font_face_hash,                     /* tp_hash */
   0,                                  /* tp_call */
   0,                                  /* tp_str */
   0,                                  /* tp_getattro */
@@ -123,7 +137,7 @@ PyTypeObject PycairoFontFace_Type = {
   0,                                  /* tp_doc */
   0,                                  /* tp_traverse */
   0,                                  /* tp_clear */
-  0,                                  /* tp_richcompare */
+  font_face_richcompare,              /* tp_richcompare */
   0,                                  /* tp_weaklistoffset */
   0,                                  /* tp_iter */
   0,                                  /* tp_iternext */
@@ -150,13 +164,18 @@ static PyObject *
 toy_font_face_new (PyTypeObject *type, PyObject *args, PyObject *kwds) {
   const char *utf8;
   PyObject *o;
-  cairo_font_slant_t slant   = CAIRO_FONT_SLANT_NORMAL;
-  cairo_font_weight_t weight = CAIRO_FONT_WEIGHT_NORMAL;
+  cairo_font_slant_t slant;
+  cairo_font_weight_t weight;
+  int slant_arg = CAIRO_FONT_SLANT_NORMAL;
+  int weight_arg = CAIRO_FONT_WEIGHT_NORMAL;
 
   if (!PyArg_ParseTuple (args, PYCAIRO_ENC_TEXT_FORMAT "|ii:ToyFontFace.__new__",
-			 "utf-8", &utf8, &slant, &weight))
+                         "utf-8", &utf8, &slant_arg, &weight_arg))
     return NULL;
 
+  slant = (cairo_font_slant_t)slant_arg;
+  weight = (cairo_font_weight_t)weight_arg;
+
   o = PycairoFontFace_FromFontFace (
 		     cairo_toy_font_face_create (utf8, slant, weight));
   PyMem_Free((void *)utf8);
@@ -164,17 +183,17 @@ toy_font_face_new (PyTypeObject *type, PyObject *args, PyObject *kwds) {
 }
 
 static PyObject *
-toy_font_get_family (PycairoToyFontFace *o) {
+toy_font_get_family (PycairoToyFontFace *o, PyObject *ignored) {
   return PYCAIRO_PyUnicode_FromString (cairo_toy_font_face_get_family (o->font_face));
 }
 
 static PyObject *
-toy_font_get_slant (PycairoToyFontFace *o) {
+toy_font_get_slant (PycairoToyFontFace *o, PyObject *ignored) {
   RETURN_INT_ENUM (FontSlant, cairo_toy_font_face_get_slant (o->font_face));
 }
 
 static PyObject *
-toy_font_get_weight (PycairoToyFontFace *o) {
+toy_font_get_weight (PycairoToyFontFace *o, PyObject *ignored) {
   RETURN_INT_ENUM (FontWeight, cairo_toy_font_face_get_weight (o->font_face));
 }
 
@@ -233,12 +252,6 @@ PyTypeObject PycairoToyFontFace_Type = {
 
 /* class cairo.ScaledFont ------------------------------------------------- */
 
-/* PycairoScaledFont_FromScaledFont
- * Create a new PycairoScaledFont from a cairo_scaled_font_t
- * scaled_font - a cairo_scaled_font_t to 'wrap' into a Python object.
- *               it is unreferenced if the PycairoScaledFont creation fails
- * Return value: New reference or NULL on failure
- */
 PyObject *
 PycairoScaledFont_FromScaledFont (cairo_scaled_font_t *scaled_font) {
   PyObject *o;
@@ -285,7 +298,7 @@ scaled_font_new (PyTypeObject *type, PyObject *args, PyObject *kwds) {
 }
 
 static PyObject *
-scaled_font_extents (PycairoScaledFont *o) {
+scaled_font_extents (PycairoScaledFont *o, PyObject *ignored) {
   cairo_font_extents_t e;
 
   cairo_scaled_font_extents (o->scaled_font, &e);
@@ -295,14 +308,14 @@ scaled_font_extents (PycairoScaledFont *o) {
 }
 
 static PyObject *
-scaled_font_get_font_face (PycairoScaledFont *o) {
+scaled_font_get_font_face (PycairoScaledFont *o, PyObject *ignored) {
   return PycairoFontFace_FromFontFace (
                cairo_font_face_reference (
 		   cairo_scaled_font_get_font_face (o->scaled_font)));
 }
 
 static PyObject *
-scaled_font_get_scale_matrix (PycairoScaledFont *o) {
+scaled_font_get_scale_matrix (PycairoScaledFont *o, PyObject *ignored) {
   cairo_matrix_t matrix;
   cairo_scaled_font_get_scale_matrix (o->scaled_font, &matrix);
   return PycairoMatrix_FromMatrix (&matrix);
@@ -310,22 +323,31 @@ scaled_font_get_scale_matrix (PycairoScaledFont *o) {
 
 static PyObject *
 scaled_font_text_extents (PycairoScaledFont *o, PyObject *args) {
-  const char *utf8;
   cairo_text_extents_t extents;
+  const char *utf8;
+  PyObject *ext_args, *res;
 
-  if (!PyArg_ParseTuple (args, PYCAIRO_ENC_TEXT_FORMAT ":ScaledFont.text_extents", "utf-8", &utf8))
+  if (!PyArg_ParseTuple (args,
+        PYCAIRO_ENC_TEXT_FORMAT ":ScaledFont.text_extents", "utf-8", &utf8))
     return NULL;
 
+  Py_BEGIN_ALLOW_THREADS;
   cairo_scaled_font_text_extents (o->scaled_font, utf8, &extents);
-  PyMem_Free((void *)utf8);
-  RETURN_NULL_IF_CAIRO_SCALED_FONT_ERROR(o->scaled_font);
-  return Py_BuildValue("(dddddd)", extents.x_bearing, extents.y_bearing,
-		       extents.width, extents.height, extents.x_advance,
-		       extents.y_advance);
+  Py_END_ALLOW_THREADS;
+
+  PyMem_Free ((void *)utf8);
+
+  RETURN_NULL_IF_CAIRO_SCALED_FONT_ERROR (o->scaled_font);
+  ext_args =  Py_BuildValue ("(dddddd)", extents.x_bearing, extents.y_bearing,
+                             extents.width, extents.height, extents.x_advance,
+                             extents.y_advance);
+  res = PyObject_Call ((PyObject *)&PycairoTextExtents_Type, ext_args, NULL);
+  Py_DECREF (ext_args);
+  return res;
 }
 
 static PyObject *
-scaled_font_get_ctm (PycairoScaledFont *o) {
+scaled_font_get_ctm (PycairoScaledFont *o, PyObject *ignored) {
   cairo_matrix_t matrix;
 
   Py_BEGIN_ALLOW_THREADS;
@@ -336,7 +358,7 @@ scaled_font_get_ctm (PycairoScaledFont *o) {
 }
 
 static PyObject *
-scaled_font_get_font_matrix (PycairoScaledFont *o) {
+scaled_font_get_font_matrix (PycairoScaledFont *o, PyObject *ignored) {
   cairo_matrix_t matrix;
 
   Py_BEGIN_ALLOW_THREADS;
@@ -347,7 +369,7 @@ scaled_font_get_font_matrix (PycairoScaledFont *o) {
 }
 
 static PyObject *
-scaled_font_get_font_options (PycairoScaledFont *o) {
+scaled_font_get_font_options (PycairoScaledFont *o, PyObject *ignored) {
   cairo_font_options_t *options = cairo_font_options_create();
 
   Py_BEGIN_ALLOW_THREADS;
@@ -357,15 +379,138 @@ scaled_font_get_font_options (PycairoScaledFont *o) {
   return PycairoFontOptions_FromFontOptions (options);
 }
 
+static PyObject *
+scaled_font_text_to_glyphs (PycairoScaledFont *o, PyObject *args) {
+  const char *utf8;
+  double x, y;
+  int with_clusters = 1;
+
+  cairo_status_t status;
+  cairo_glyph_t *glyphs = NULL;
+  int num_glyphs;
+  cairo_text_cluster_t *clusters = NULL;
+  int num_clusters;
+  cairo_text_cluster_flags_t cluster_flags;
+
+  int i;
+  PyObject *glyph_list = NULL;
+  PyObject *cluster_list = NULL;
+  PyObject *flags = NULL;
+  PyObject *pyglyph, *glyph_args;
+  PyObject *pycluster, *cluster_args;
+
+  if (!PyArg_ParseTuple (args,
+      "dd" PYCAIRO_ENC_TEXT_FORMAT "|i:ScaledFont.text_to_glyphs",
+      &x, &y, "utf-8", &utf8, &with_clusters))
+    return NULL;
+
+  Py_BEGIN_ALLOW_THREADS;
+  status = cairo_scaled_font_text_to_glyphs (
+    o->scaled_font,
+    x, y,
+    utf8, -1,
+    &glyphs, &num_glyphs,
+    (with_clusters) ? &clusters : NULL,
+    (with_clusters) ? &num_clusters : NULL,
+    (with_clusters) ? &cluster_flags : NULL);
+  Py_END_ALLOW_THREADS;
+
+  PyMem_Free ((void *)utf8);
+  RETURN_NULL_IF_CAIRO_ERROR (status);
+
+  glyph_list = PyList_New (num_glyphs);
+  if (glyph_list == NULL)
+    goto error;
+  for(i=0; i < num_glyphs; i++) {
+    cairo_glyph_t *glyph = &glyphs[i];
+    glyph_args = Py_BuildValue(
+      "(kdd)", glyph->index, glyph->x, glyph->y);
+    if (glyph_args == NULL)
+      goto error;
+    pyglyph = PyObject_Call(
+      (PyObject *)&PycairoGlyph_Type, glyph_args, NULL);
+    if (pyglyph == NULL) {
+      Py_DECREF (glyph_args);
+      goto error;
+    }
+    PyList_SET_ITEM (glyph_list, i, pyglyph);
+  }
+  cairo_glyph_free (glyphs);
+  glyphs = NULL;
+
+  if (with_clusters) {
+    cluster_list = PyList_New (num_clusters);
+    if (cluster_list == NULL)
+      goto error;
+    for(i=0; i < num_clusters; i++) {
+      cairo_text_cluster_t *cluster = &clusters[i];
+      cluster_args = Py_BuildValue(
+        "(ii)", cluster->num_bytes, cluster->num_glyphs);
+      if (cluster_args == NULL)
+        goto error;
+      pycluster = PyObject_Call(
+        (PyObject *)&PycairoTextCluster_Type, cluster_args, NULL);
+      if (pycluster == NULL) {
+        Py_DECREF (cluster_args);
+        goto error;
+      }
+      PyList_SET_ITEM (cluster_list, i, pycluster);
+    }
+    cairo_text_cluster_free (clusters);
+    clusters = NULL;
+
+    flags = CREATE_INT_ENUM (TextClusterFlags, cluster_flags);
+    if (flags == NULL)
+      goto error;
+
+    return Py_BuildValue ("(NNN)", glyph_list, cluster_list, flags);
+  } else {
+    return glyph_list;
+  }
+error:
+  cairo_glyph_free (glyphs);
+  cairo_text_cluster_free (clusters);
+  Py_XDECREF (glyph_list);
+  Py_XDECREF (cluster_list);
+  Py_XDECREF (flags);
+  return NULL;
+}
+
+static PyObject *
+scaled_font_glyph_extents (PycairoScaledFont *o, PyObject *args) {
+  int num_glyphs = -1;
+  cairo_glyph_t *glyphs;
+  cairo_text_extents_t extents;
+  PyObject *py_object, *ext_args, *res;
+
+  if (!PyArg_ParseTuple (args, "O|i:ScaledFont.glyph_extents",
+      &py_object, &num_glyphs))
+    return NULL;
+
+  glyphs = _PycairoGlyphs_AsGlyphs (py_object, &num_glyphs);
+  if (glyphs == NULL)
+    return NULL;
+
+  Py_BEGIN_ALLOW_THREADS
+  cairo_scaled_font_glyph_extents (
+    o->scaled_font, glyphs, num_glyphs, &extents);
+  Py_END_ALLOW_THREADS;
+
+  PyMem_Free (glyphs);
+  RETURN_NULL_IF_CAIRO_SCALED_FONT_ERROR (o->scaled_font);
+  ext_args = Py_BuildValue ("(dddddd)", extents.x_bearing, extents.y_bearing,
+                            extents.width, extents.height, extents.x_advance,
+                            extents.y_advance);
+  res = PyObject_Call ((PyObject *)&PycairoTextExtents_Type, ext_args, NULL);
+  Py_DECREF (ext_args);
+  return res;
+}
+
 static PyMethodDef scaled_font_methods[] = {
   /* methods never exposed in a language binding:
    * cairo_scaled_font_destroy()
    * cairo_scaled_font_get_type()
    * cairo_scaled_font_reference()
-   *
-   * TODO if requested:
-   * cairo_scaled_font_glyph_extents
-   * cairo_scaled_font_text_to_glyphs
    */
   {"extents",       (PyCFunction)scaled_font_extents,        METH_NOARGS},
   {"get_font_face", (PyCFunction)scaled_font_get_font_face,  METH_NOARGS},
@@ -374,9 +519,31 @@ static PyMethodDef scaled_font_methods[] = {
   {"get_font_options", (PyCFunction)scaled_font_get_font_options, METH_NOARGS},
   {"get_scale_matrix", (PyCFunction)scaled_font_get_scale_matrix, METH_VARARGS},
   {"text_extents",  (PyCFunction)scaled_font_text_extents,   METH_VARARGS},
+  {"text_to_glyphs",  (PyCFunction)scaled_font_text_to_glyphs,    METH_VARARGS},
+  {"glyph_extents", (PyCFunction)scaled_font_glyph_extents,  METH_VARARGS},
   {NULL, NULL, 0, NULL},
 };
 
+static PyObject*
+scaled_font_richcompare (PyObject *self, PyObject *other, int op)
+{
+  if (Py_TYPE(self) == Py_TYPE(other))
+    return Pycairo_richcompare (
+      ((PycairoScaledFont *)self)->scaled_font,
+      ((PycairoScaledFont *)other)->scaled_font,
+      op);
+  else {
+    Py_INCREF(Py_NotImplemented);
+    return Py_NotImplemented;
+  }
+}
+
+static PYCAIRO_Py_hash_t
+scaled_font_hash (PyObject *self)
+{
+  return PYCAIRO_Py_hash_t_FromVoidPtr (
+    ((PycairoScaledFont *)self)->scaled_font);
+}
 
 PyTypeObject PycairoScaledFont_Type = {
   PyVarObject_HEAD_INIT(NULL, 0)
@@ -392,7 +559,7 @@ PyTypeObject PycairoScaledFont_Type = {
   0,                                  /* tp_as_number */
   0,                                  /* tp_as_sequence */
   0,                                  /* tp_as_mapping */
-  0,                                  /* tp_hash */
+  scaled_font_hash,                   /* tp_hash */
   0,                                  /* tp_call */
   0,                                  /* tp_str */
   0,                                  /* tp_getattro */
@@ -402,7 +569,7 @@ PyTypeObject PycairoScaledFont_Type = {
   0,                                  /* tp_doc */
   0,                                  /* tp_traverse */
   0,                                  /* tp_clear */
-  0,                                  /* tp_richcompare */
+  scaled_font_richcompare,            /* tp_richcompare */
   0,                                  /* tp_weaklistoffset */
   0,                                  /* tp_iter */
   0,                                  /* tp_iternext */
@@ -425,12 +592,6 @@ PyTypeObject PycairoScaledFont_Type = {
 
 /* class cairo.FontOptions ------------------------------------------------ */
 
-/* PycairoFontOptions_FromFontOptions
- * Create a new PycairoFontOptions from a cairo_font_options_t
- * font_options - a cairo_font_options_t to 'wrap' into a Python object.
- *                it is unreferenced if the PycairoFontOptions creation fails
- * Return value: New reference or NULL on failure
- */
 PyObject *
 PycairoFontOptions_FromFontOptions (cairo_font_options_t *font_options) {
   PyObject *o;
@@ -461,82 +622,145 @@ font_options_dealloc(PycairoFontOptions *o) {
 
 static PyObject *
 font_options_new (PyTypeObject *type, PyObject *args, PyObject *kwds) {
+
+  if (!PyArg_ParseTuple (args, ":FontOptions.__new__"))
+    return NULL;
+
   return PycairoFontOptions_FromFontOptions (cairo_font_options_create());
 }
 
+#if CAIRO_VERSION >= CAIRO_VERSION_ENCODE(1, 15, 12)
 static PyObject *
-font_options_get_antialias (PycairoFontOptions *o) {
+font_options_set_variations (PycairoFontOptions *o, PyObject *args) {
+    const char *variations;
+    PyObject *maybe_none;
+
+    if (!PyArg_ParseTuple (args, "O:FontOptions.set_variations", &maybe_none))
+        return NULL;
+
+    if (maybe_none != Py_None) {
+        if (!PyArg_ParseTuple (args,
+                PYCAIRO_ENC_TEXT_FORMAT ":FontOptions.set_variations", "utf-8",
+                &variations))
+            return NULL;
+    } else {
+        variations = NULL;
+    }
+
+    Py_BEGIN_ALLOW_THREADS;
+    cairo_font_options_set_variations (o->font_options, variations);
+    Py_END_ALLOW_THREADS;
+
+    if (variations != NULL)
+        PyMem_Free((void *)variations);
+
+    RETURN_NULL_IF_CAIRO_FONT_OPTIONS_ERROR(o->font_options);
+    Py_RETURN_NONE;
+}
+
+static PyObject *
+font_options_get_variations (PycairoFontOptions *o, PyObject *ignored) {
+    const char *variations;
+
+    Py_BEGIN_ALLOW_THREADS;
+    variations = cairo_font_options_get_variations (o->font_options);
+    Py_END_ALLOW_THREADS;
+
+    if (variations == NULL)
+        Py_RETURN_NONE;
+
+    return PYCAIRO_PyUnicode_FromString (variations);
+}
+#endif
+
+static PyObject *
+font_options_get_antialias (PycairoFontOptions *o, PyObject *ignored) {
   RETURN_INT_ENUM (Antialias, cairo_font_options_get_antialias (o->font_options));
 }
 
 static PyObject *
-font_options_get_hint_metrics (PycairoFontOptions *o) {
+font_options_get_hint_metrics (PycairoFontOptions *o, PyObject *ignored) {
   RETURN_INT_ENUM (HintMetrics,
                    cairo_font_options_get_hint_metrics (o->font_options));
 }
 
 static PyObject *
-font_options_get_hint_style (PycairoFontOptions *o) {
+font_options_get_hint_style (PycairoFontOptions *o, PyObject *ignored) {
   RETURN_INT_ENUM (HintStyle,
                    cairo_font_options_get_hint_style (o->font_options));
 }
 
 static PyObject *
-font_options_get_subpixel_order (PycairoFontOptions *o) {
+font_options_get_subpixel_order (PycairoFontOptions *o, PyObject *ignored) {
   RETURN_INT_ENUM (SubpixelOrder,
                    cairo_font_options_get_subpixel_order (o->font_options));
 }
 
 static PyObject *
 font_options_set_antialias (PycairoFontOptions *o, PyObject *args) {
-  cairo_antialias_t aa = CAIRO_ANTIALIAS_DEFAULT;
+  cairo_antialias_t antialias;
+  int antialias_arg = CAIRO_ANTIALIAS_DEFAULT;
 
-  if (!PyArg_ParseTuple(args, "|i:FontOptions.set_antialias", &aa))
+  if (!PyArg_ParseTuple (args, "|i:FontOptions.set_antialias", &antialias_arg))
     return NULL;
 
-  cairo_font_options_set_antialias (o->font_options, aa);
+  antialias = (cairo_antialias_t)antialias_arg;
+
+  cairo_font_options_set_antialias (o->font_options, antialias);
   RETURN_NULL_IF_CAIRO_FONT_OPTIONS_ERROR(o->font_options);
   Py_RETURN_NONE;
 }
 
 static PyObject *
 font_options_set_hint_metrics (PycairoFontOptions *o, PyObject *args) {
-  cairo_hint_metrics_t hm = CAIRO_HINT_METRICS_DEFAULT;
+  cairo_hint_metrics_t hint_metrics;
+  int hint_metrics_arg = CAIRO_HINT_METRICS_DEFAULT;
 
-  if (!PyArg_ParseTuple(args, "|i:FontOptions.set_hint_metrics", &hm))
+  if (!PyArg_ParseTuple (args, "|i:FontOptions.set_hint_metrics",
+                         &hint_metrics_arg))
     return NULL;
 
-  cairo_font_options_set_hint_metrics (o->font_options, hm);
+  hint_metrics = (cairo_hint_metrics_t)hint_metrics_arg;
+
+  cairo_font_options_set_hint_metrics (o->font_options, hint_metrics);
   RETURN_NULL_IF_CAIRO_FONT_OPTIONS_ERROR(o->font_options);
   Py_RETURN_NONE;
 }
 
 static PyObject *
 font_options_set_hint_style (PycairoFontOptions *o, PyObject *args) {
-  cairo_hint_style_t hs = CAIRO_HINT_STYLE_DEFAULT;
+  cairo_hint_style_t hint_style;
+  int hint_style_arg = CAIRO_HINT_STYLE_DEFAULT;
 
-  if (!PyArg_ParseTuple(args, "|i:FontOptions.set_hint_style", &hs))
+  if (!PyArg_ParseTuple (args, "|i:FontOptions.set_hint_style",
+                         &hint_style_arg))
     return NULL;
 
-  cairo_font_options_set_hint_style (o->font_options, hs);
+  hint_style = (cairo_hint_style_t)hint_style_arg;
+
+  cairo_font_options_set_hint_style (o->font_options, hint_style);
   RETURN_NULL_IF_CAIRO_FONT_OPTIONS_ERROR(o->font_options);
   Py_RETURN_NONE;
 }
 
 static PyObject *
 font_options_set_subpixel_order (PycairoFontOptions *o, PyObject *args) {
-  cairo_subpixel_order_t so = CAIRO_SUBPIXEL_ORDER_DEFAULT;
+  cairo_subpixel_order_t subpixel_order;
+  int subpixel_order_arg = CAIRO_SUBPIXEL_ORDER_DEFAULT;
 
-  if (!PyArg_ParseTuple(args, "|i:FontOptions.set_subpixel_order", &so))
+  if (!PyArg_ParseTuple (args, "|i:FontOptions.set_subpixel_order",
+                         &subpixel_order_arg))
     return NULL;
 
-  cairo_font_options_set_subpixel_order (o->font_options, so);
+  subpixel_order = (cairo_subpixel_order_t)subpixel_order_arg;
+
+  cairo_font_options_set_subpixel_order (o->font_options, subpixel_order);
   RETURN_NULL_IF_CAIRO_FONT_OPTIONS_ERROR(o->font_options);
   Py_RETURN_NONE;
 }
 
 static PyObject *
-font_options_copy (PycairoFontOptions *o) {
+font_options_copy (PycairoFontOptions *o, PyObject *ignored) {
   cairo_font_options_t *new;
 
   Py_BEGIN_ALLOW_THREADS;
@@ -547,7 +771,7 @@ font_options_copy (PycairoFontOptions *o) {
 }
 
 static PyObject *
-font_options_hash (PycairoFontOptions *o) {
+font_options_hash (PycairoFontOptions *o, PyObject *ignored) {
   unsigned long hash;
 
   Py_BEGIN_ALLOW_THREADS;
@@ -593,6 +817,10 @@ static PyMethodDef font_options_methods[] = {
    * cairo_font_options_destroy()
    * cairo_font_options_reference()
    */
+#if CAIRO_VERSION >= CAIRO_VERSION_ENCODE(1, 15, 12)
+  {"get_variations",    (PyCFunction)font_options_get_variations, METH_NOARGS},
+  {"set_variations",    (PyCFunction)font_options_set_variations, METH_VARARGS},
+#endif
   {"get_antialias",     (PyCFunction)font_options_get_antialias,  METH_NOARGS},
   {"get_hint_metrics",  (PyCFunction)font_options_get_hint_metrics,
    METH_NOARGS},
@@ -650,7 +878,7 @@ PyTypeObject PycairoFontOptions_Type = {
   0,                                  /* tp_as_number */
   0,                                  /* tp_as_sequence */
   0,                                  /* tp_as_mapping */
-  0,                                  /* tp_hash */
+  (hashfunc)PyObject_HashNotImplemented,/* tp_hash */
   0,                                  /* tp_call */
   0,                                  /* tp_str */
   0,                                  /* tp_getattro */
